package conf

import (
	"fmt"
	"os"
	"strconv"
	"strings"

	"sku3dweb/log"

	"github.com/spf13/viper"
)

type TaskConfig struct {
	Name              string
	Broker            string
	DefaultQueue      string
	ResultBackend     string
	ResultsExpireIn   int32
	AmqpExchange      string
	AmqpExchangeType  string
	AmqpBindingKey    string
	AmqpPrefetchCount int32
}

type Proxy struct {
	Prefix string
	Host   string
	Scheme string
	Path   string
	Name   string
}

type AppConf struct {
	Etcd struct {
		Endpoints   []string
		UserName    string
		Password    string
		DialTimeout int64
	}
	Mysql struct {
		DSN string
	}
	Redis struct {
		Addr     string
		Password string // no password set
		Db       int    // use default DB
	}
	Log struct {
		FileName    string
		Level       int32
		ServiceName string
	}
	Port  int32
	Mongo struct {
		DSN      string
		Database string
	}

	Debug struct {
		UserId    string
		UserPhone string
		UserRole  string
	}
	Obs struct {
		Bucket string
	}

	Minio struct {
		Bucket   string
		PubHost  string
		Endpoint string

		AccessKey string
		AccessSec string
	}
	Proxies []*Proxy //代理连接

	TaskRender        TaskConfig
	TaskUploadProcess TaskConfig
	TaskShadow        TaskConfig
	TaskCenter        string
	SearchImageAddr   string

	Nats struct {
		Url               string
		MaxReconnect      int
		ReconnDelaySecond int

		OsgconvStreamTopic string
		ShadowStreamTopic  string
		HdrProcStreamTopic string
		RenderStreamTopic  string
	}

	TreeHost string `json:"treeHost"`
}

func LoadConfFile(filepath string) (*AppConf, error) {
	file, err := os.Open(filepath)
	if err != nil {
		log.Errorf("can not open file:%s", filepath)
		return nil, err
	}
	v := viper.New()
	v.SetConfigType("yaml")
	err = v.ReadConfig(file)
	if err != nil {
		return nil, err
	}
	c := new(AppConf)
	err = v.Unmarshal(c)
	return c, err
}

func NewAppConf(filePath string) (*AppConf, error) {

	c, err := LoadConfFile(filePath)
	if err != nil {
		return c, err
	}

	//初始化log
	_ = log.NewLoggerSugar(c.Log.ServiceName, c.Log.FileName, c.Log.Level)

	//环境变量
	mongo := os.Getenv("MONGO")
	fmt.Println("env MONGO=>", mongo)

	if len(mongo) > 0 {
		c.Mongo.DSN = mongo
		//解析出database mongodb://admin:123456@124.70.148.113:27018/sku3d_dadong?authSource=admin
		start := strings.LastIndex(mongo, "/")
		end := strings.LastIndex(mongo, "?")
		database := mongo[start+1 : end]
		if len(database) > 0 {
			c.Mongo.Database = database
		}
		fmt.Println(c.Mongo.DSN, c.Mongo.Database)
	}
	//redis
	redis := os.Getenv("REDIS")
	fmt.Println("Env REDIS=>", redis)

	if len(redis) > 0 {
		c.Redis.Addr = redis
		start := strings.LastIndex(redis, "/")
		if start > 0 {
			db := redis[start:]
			dbint, _ := strconv.Atoi(db)
			c.Redis.Db = dbint
			c.Redis.Addr = string(redis[0:start])
		}

		fmt.Println(c.Redis.Addr, c.Redis.Db)
	}
	//rabbitmq
	nats := os.Getenv("NATS")
	if len(nats) > 0 {
		c.Nats.Url = nats
	}

	fmt.Println("Env NATS=>", nats)

	//tree
	tree := os.Getenv("TREE")
	if len(tree) > 0 {
		c.TreeHost = tree
	}
	searchImageAddr := os.Getenv("SEARCH_IMAGE_ADDR")
	if len(searchImageAddr) > 0 {
		c.SearchImageAddr = searchImageAddr
	}
	fmt.Println("search image addr=>", searchImageAddr)
	AppConfig = c

	return c, nil
}

var AppConfig *AppConf = nil
